import {
  Column,
  Entity,
  JoinColumn,
  ManyToOne,
  OneToMany,
  PrimaryGeneratedColumn,
} from 'typeorm';
import { CarritoMotivo } from '../../carrito-motivo/entity/carrito-motivo.entity';
import { Equipo } from '../../equipo/entity/equipo.entity';
import { Marca } from '../../marca/entity/marca.entity';
import { Modelo } from '../../modelo/entity/modelo.entity';
import { Modulo } from '../../modulo/entity/modulo.entity';
import { TipoCarrito } from '../../institucion-tipo-carrito/entity/tipo-carrito.entity';

@Entity()
export class Carrito {
  @PrimaryGeneratedColumn()
  id_carrito: number;

  @Column({ type: Boolean, nullable: false, default: false })
  activo: boolean;

  @Column({ type: String, nullable: false, length: 4 })
  carrito: string;

  @Column({ type: Number, nullable: true })
  id_marca: number;

  @Column({ type: Number, nullable: true })
  id_modelo: number;

  @Column({ type: Number, nullable: true })
  id_modulo: number;

  @Column({ type: Number, nullable: true })
  id_tipo_carrito: number;

  @ManyToOne(() => Marca, (marca) => marca.carritos)
  @JoinColumn({ name: 'id_marca' })
  marca: Marca;

  @ManyToOne(() => Modelo, (modelo) => modelo.carritos)
  @JoinColumn({ name: 'id_modelo' })
  modelo: Modelo;

  @ManyToOne(() => Modulo, (modulo) => modulo.carritos)
  @JoinColumn({ name: 'id_modulo' })
  modulo: Modulo;

  @ManyToOne(() => TipoCarrito, (tipoCarrito) => tipoCarrito.carritos)
  @JoinColumn({ name: 'id_tipo_carrito' })
  tipoCarrito: TipoCarrito;

  @OneToMany(() => CarritoMotivo, (motivo) => motivo.carrito)
  motivos: CarritoMotivo[];

  @OneToMany(() => Equipo, (equipo) => equipo.carrito)
  equipos: Equipo[];
}
