import { Injectable } from '@nestjs/common';
import { Cron } from '@nestjs/schedule';
import { AppGateway } from '../app.gateway';
import { EquipoService } from '../equipo/equipo.service';
import { InstitucionUsuarioService } from '../institucion-usuario/institucion-usuario.service';
import { MultaService } from '../multa/multa.service';
import { PrestamoService } from '../prestamo/prestamo.service';

@Injectable()
export class CronService {
  constructor(
    private appGateway: AppGateway,
    private equipoService: EquipoService,
    private multaService: MultaService,
    private prestamoService: PrestamoService,
    private institucionUsuarioService: InstitucionUsuarioService,
  ) {}

  @Cron('* 6-20 * * 1-6', { name: 'prestamos_desactivacion' })
  async desactivarPrestamos() {
    await this.prestamoService
      .findAllEquiposRepetidos()
      .then(async (prestamos) => {
        // Recorremos el array de prestamos
        for (let i = 0; i < prestamos.length; i++) {
          // Desactivamos préstamo y registramos que fue por repetición de equipo
          prestamos[i].activo = false;
          prestamos[i].cancelado_repeticion = true;
          // Guardamos cambios
          await this.prestamoService.save(prestamos[i]).then((_) => {
            // Actualizamos las interfaces del los usuarios
            this.appGateway.actualizarOperador(
              prestamos[i].equipo.carrito.modulo.institucion.id_institucion,
            );
            this.appGateway.actualizarUsuario(prestamos[i].usuario.id_usuario);
          });
        }
      });
    await this.prestamoService
      .findAllActivosExpirados()
      .then(async (prestamos) => {
        // Recorremos el array de prestamos
        for (let i = 0; i < prestamos.length; i++) {
          // Desactivamos préstamO y reseteamos campos
          prestamos[i].activo = false;
          prestamos[i].cancelado_operador = true;
          prestamos[i].equipo.u = null;
          prestamos[i].equipo.status.id_status = 1;
          // Guardamos cambios
          await this.prestamoService
            .save(prestamos[i])
            .then((_) => this.equipoService.updateStatus(prestamos[i].equipo))
            .then((_) => {
              // Actualizamos las interfaces del los usuarios
              this.appGateway.actualizarOperador(
                prestamos[i].equipo.carrito.modulo.institucion.id_institucion,
              );
              this.appGateway.actualizarUsuario(
                prestamos[i].usuario.id_usuario,
              );
            });
        }
      });
  }

  @Cron('0 0 * * 1-6', { name: 'equipos_reseteo' })
  resetearEquipos() {
    return this.equipoService.findAllResetear().then(async (equipos) => {
      // Recorremos el array de equipos
      for (let i = 0; i < equipos.length; i++) {
        // Reseteamos atributo prestado
        equipos[i].prestado = false;
        // Guardamos cambios
        await this.equipoService.save(equipos[i]);
      }
    });
  }

  @Cron('0 1 * * 1-6', { name: 'multas_reseteo' })
  revisarMultas() {
    return this.multaService.findAllActivasExpiradas().then(async (multas) => {
      // Recorremos el array de multas
      for (let i = 0; i < multas.length; i++) {
        // Desacrtivamos la multa
        multas[i].activo = false;
        // Guardamos cambios
        await this.multaService.save(multas[i]).then((_) =>
          // Quitamos multas en institución usuario
          this.institucionUsuarioService.updateMulta(
            multas[i].institucionInfraccion.institucion,
            multas[i].prestamo.usuario,
            false,
          ),
        );
      }
    });
  }
}
